Package: bartMachine
Type: Package
Title: Bayesian Additive Regression Trees
Version: 1.4.2
Date: 2026-05-03
Maintainer: Adam Kapelner <kapelner@qc.cuny.edu>
Authors@R: c(person(given = "Adam",
                  family = "Kapelner",
                  role = c("aut", "cre"),
                  email = "kapelner@qc.cuny.edu",
                  comment = c(ORCID = "0000-0001-5985-6792")),
             person(given = "Justin",
                      family = "Bleich",
                      role = "aut"))
Description: An advanced implementation of Bayesian Additive Regression Trees with expanded features for data analysis and visualization.
License: GPL-3
Encoding: UTF-8
Depends: R (>= 2.14.0)
Imports: bartMachineJARs (>= 1.2.2), rJava, randomForest, missForest,
        stats, ggplot2, grid, checkmate, matrixStats, grDevices
Suggests: testthat (>= 3.0.0), withr
SystemRequirements: Java (>= 21.0), TornadoVM (if you want GPU
        optimization)
BugReports: https://github.com/kapelner/bartMachine/issues
RoxygenNote: 7.3.3
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-05-03 21:07:08 UTC; kapelner
Author: Adam Kapelner [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-5985-6792>),
  Justin Bleich [aut]
Repository: CRAN
Date/Publication: 2026-05-04 05:20:02 UTC
Built: R 4.5.2; ; 2026-05-04 07:47:56 UTC; unix
