# Generated by roxygen2: do not edit by hand

S3method(predict,bartMachine)
S3method(print,bartMachine)
S3method(summary,bartMachine)
export(bartMachine)
export(bartMachineArr)
export(bartMachineCV)
export(bart_machine_get_posterior)
export(bart_machine_num_cores)
export(bart_predict_for_test_data)
export(build_bart_machine)
export(build_bart_machine_cv)
export(calc_credible_intervals)
export(calc_prediction_intervals)
export(check_bart_error_assumptions)
export(cov_importance_test)
export(dummify_data)
export(extract_raw_node_data)
export(get_projection_weights)
export(get_sigsqs)
export(get_var_counts_over_chain)
export(get_var_props_over_chain)
export(interaction_investigator)
export(investigate_var_importance)
export(k_fold_cv)
export(linearity_test)
export(node_prediction_training_data_indices)
export(pd_plot)
export(plot_convergence_diagnostics)
export(plot_y_vs_yhat)
export(predict_bartMachineArr)
export(rmse_by_num_trees)
export(set_bart_machine_num_cores)
export(var_selection_by_permute)
export(var_selection_by_permute_cv)
import(bartMachineJARs)
import(checkmate)
import(grid)
import(missForest)
import(randomForest)
import(stats)
importFrom(grDevices,rgb)
importFrom(rJava,.jarray)
importFrom(rJava,.jcache)
importFrom(rJava,.jcall)
importFrom(rJava,.jcast)
importFrom(rJava,.jevalArray)
importFrom(rJava,.jnew)
importFrom(rJava,.jnull)
importFrom(rJava,.jpackage)
importFrom(rJava,is.jnull)
importFrom(utils,packageVersion)
importFrom(utils,tail)
