benchmark_registry_version <- "1.1"

benchmark_registry <- list(
  list(name = "mtcars", package = "datasets", response = "mpg", task = "regression"),
  list(name = "airquality", package = "datasets", response = "Ozone", task = "regression"),
  list(name = "ToothGrowth", package = "datasets", response = "len", task = "regression"),
  list(name = "warpbreaks", package = "datasets", response = "breaks", task = "regression"),
  list(name = "swiss", package = "datasets", response = "Fertility", task = "regression"),
  list(name = "trees", package = "datasets", response = "Volume", task = "regression"),
  list(name = "faithful", package = "datasets", response = "eruptions", task = "regression"),
  list(name = "rock", package = "datasets", response = "perm", task = "regression"),
  list(name = "attitude", package = "datasets", response = "rating", task = "regression"),
  list(name = "USArrests", package = "datasets", response = "Murder", task = "regression"),
  list(name = "quakes", package = "datasets", response = "mag", task = "regression"),
  list(name = "CO2", package = "datasets", response = "uptake", task = "regression"),
  list(name = "infert", package = "datasets", response = "case", task = "classification"),
  list(name = "Boston", package = "MASS", response = "medv", task = "regression"),
  list(name = "Pima.tr", package = "MASS", response = "type", task = "classification"),
  list(name = "biopsy", package = "MASS", response = "class", task = "classification", drop = c("ID")),
  list(name = "birthwt", package = "MASS", response = "low", task = "classification"),
  list(name = "cats", package = "MASS", response = "Hwt", task = "regression"),
  list(name = "cement", package = "MASS", response = "y", task = "regression"),
  list(name = "cpus", package = "MASS", response = "perf", task = "regression", drop = c("name")),
  list(name = "Cars93", package = "MASS", response = "Price", task = "regression"),
  list(name = "BostonHousing", package = "mlbench", response = "medv", task = "regression"),
  list(name = "BostonHousing2", package = "mlbench", response = "medv", task = "regression"),
  list(name = "PimaIndiansDiabetes", package = "mlbench", response = "diabetes", task = "classification"),
  list(name = "PimaIndiansDiabetes2", package = "mlbench", response = "diabetes", task = "classification"),
  list(name = "BreastCancer", package = "mlbench", response = "Class", task = "classification", drop = c("Id")),
  list(name = "HouseVotes84", package = "mlbench", response = "Class", task = "classification"),
  list(name = "Ionosphere", package = "mlbench", response = "Class", task = "classification"),
  list(name = "Sonar", package = "mlbench", response = "Class", task = "classification"),
  list(name = "Servo", package = "mlbench", response = "Class", task = "regression"),
  list(name = "Auto", package = "ISLR", response = "mpg", task = "regression", drop = c("name")),
  list(name = "Carseats", package = "ISLR", response = "Sales", task = "regression"),
  list(name = "College", package = "ISLR", response = "Apps", task = "regression"),
  list(name = "Credit", package = "ISLR", response = "Balance", task = "regression", drop = c("ID")),
  list(name = "Hitters", package = "ISLR", response = "Salary", task = "regression", drop = c("Player")),
  list(name = "Portfolio", package = "ISLR", response = "Y", task = "regression"),
  list(name = "Wage", package = "ISLR", response = "wage", task = "regression"),
  list(name = "Default", package = "ISLR", response = "default", task = "classification"),
  list(name = "OJ", package = "ISLR", response = "Purchase", task = "classification"),
  list(name = "Weekly", package = "ISLR", response = "Direction", task = "classification"),
  list(name = "Smarket", package = "ISLR", response = "Direction", task = "classification"),
  list(name = "Caravan", package = "ISLR", response = "Purchase", task = "classification", tags = c("large"))
)
