#' bartMachine: Bayesian Additive Regression Trees
#'
#' @importFrom rJava .jpackage .jcall .jnew .jevalArray .jarray is.jnull .jcache .jcast .jnull
#' @import bartMachineJARs
#' @import randomForest
#' @import missForest
#' @import stats
#' @importFrom utils packageVersion tail
#' @importFrom grDevices rgb
#' @import grid
#' @import checkmate
#' @keywords internal
"_PACKAGE"
